package com.chelvc.cloud.vehicle.api.service;
import java.util.List;
import java.util.Map;

import com.chelvc.cloud.vehicle.api.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDetailDTO;
import com.chelvc.cloud.vehicle.api.param.*;
import com.chelvc.framework.common.model.Pagination;

/**
 * 商品业务接口
 *
 * @author Woody
 * @date 2023/8/6
 */
public interface GoodsService {
    /**
     * 获取商品详情
     *
     * @param id 商品ID
     * @return 商品详情
     */
    GoodsDetailDTO getGoodsDetail(Long id);

    /**
     * 查询商家商品
     *
     * @param merchantId 商家ID
     * @param param      查询参数
     * @return 商品列表
     */
    List<GoodsDTO> listMerchantSimpleGoods(Long merchantId, GoodsQueryParam param);


    /**
     * 新增商品
     *
     * @param param 新增参数
     * @return 商品主键
     */
    Long addGoods(GoodsModifyParam param);

    /**
     * 修改商品
     *
     * @param id    商品主键
     * @param param 修改参数
     */
    void updateGoods(Long id, GoodsModifyParam param);

    /**
     * 删除商品
     *
     * @param id    商品主键
     */
    void deleteGoods(Long id);

    /**
     * 查询商品
     *
     * @param param 查询参数
     * @return 商品分页信息
     */
    Pagination<GoodsDTO> getGoodsPaging(GoodsPagingParam param);

    /**
     * 修改商品状态
     *
     * @param ids    商品ID集合
     * @param status 商品状态
     */
    void updateGoodsStatus(List<Long> ids, GoodsStatus status);

    /**
     * 商家端商品管理
     * @param param
     * @return
     */
    Map<String,Object> getCategoryAndGoods(GoodsQueryParam param);

    /**
     * 获取商品的订单量以及交易金额
     * @return
     */
    List<Map<String,Object>> getGoodsAmount();


}
