package com.chelvc.cloud.vehicle.server.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantRankCountDTO;
import com.chelvc.cloud.vehicle.server.entity.Merchant;

import java.math.BigDecimal;
import java.util.List;

/**
 * 商家业务操作接口
 *
 * @author Woody
 * @date 2023/5/2
 */
public interface MerchantService extends IService<Merchant> {
    /**
     * 获取商家id
     * @param userId 用户id
     * @return
     */
    List<Long> getMerchantIdsByUserId(Long userId);

    /**
     * 商家信息
     * @param merchantId
     * @return
     */
    MerchantDTO getMerchantById(Long merchantId);

    /**
     * 获取商家名称
     */
    String getMerchantName(Long merchantId);


    /**
     * 查询商家排名
     * @param regionId
     * @param limit
     * @param level
     * @return
     */
    List<MerchantRankCountDTO> rankMerchants(Integer regionId, Integer limit, Integer level, Long categoryId);

    /**
     * 根据地区查询上个月所有商家的平台抽成金额
     * @param regionId
     * @param level
     * @return
     */
    BigDecimal queryTotalCommissionAmountByLastMonth(Integer regionId, Integer level, Long categoryId);


    /**
     * 查询用户商家信息
     * @param userId
     * @return
     */
    MerchantDTO getMerchantByUserId(Long userId);


    List<MerchantDTO> getMerchantByIds(List<Long> ids);
}
