package com.chelvc.cloud.vehicle.server.service.impl;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.model.Scope;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.api.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.api.param.MerchantAuthPagingParam;
import com.chelvc.cloud.vehicle.server.copier.MerchantAuthCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantAuthMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantAuth;
import com.chelvc.cloud.vehicle.server.service.MerchantAuthService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 商家认证业务操作实现
 *
 * @author liude
 * @date 2023/12/2
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.MerchantAuthService.class)
public class MerchantAuthServiceImpl extends ServiceImpl<MerchantAuthMapper, MerchantAuth> implements MerchantAuthService,
        com.chelvc.cloud.vehicle.api.service.MerchantAuthService {
    private final MerchantService merchantService;

    private final UserService userService;
    @Override
    public Long addMerchantAuth(@NonNull MerchantAuthModifyParam param) {
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = MerchantAuthCopier.INSTANCE.copying(param);
        Integer count = this.lambdaQuery().eq(MerchantAuth::getApplicant, userId).count();
        AssertUtils.available(count <= 0, "商家认证已存在");
        merchantAuth.setApplicationTime(new Date());
        merchantAuth.setApplicant(userId);
        this.save(merchantAuth);
        return merchantAuth.getId();
    }

    @Override
    public void updateMerchantAuth(@NonNull Long id, @NonNull MerchantAuthModifyParam param) {
        MerchantAuth merchantAuth = AssertUtils.available(this.getById(id), "商家认证不存在");
        MerchantAuthCopier.INSTANCE.copying(param, merchantAuth);
        this.updateById(merchantAuth);
    }

    @Override
    public Pagination<MerchantAuthDTO> getMerchantAuthPaging(@NonNull MerchantAuthPagingParam param) {
        Long userId = null;
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            userId = SessionContextHolder.getId();
        }
        Page<MerchantAuth> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getStoreName()), MerchantAuth::getStoreName, param.getStoreName())
                .eq(StringUtils.notEmpty(userId),MerchantAuth::getApplicant,userId)
                .orderByAsc(MerchantAuth::getReviewTime).page(DatabaseContextHolder.page(param.getPaging()));
        return DatabaseContextHolder.pagination(page, MerchantAuthCopier.INSTANCE::copying);
    }

    @Override
    public MerchantAuthDTO queryById() {
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = this.lambdaQuery()
                .eq(MerchantAuth::getApplicant, userId).one();
        MerchantAuthDTO merchantAuthDTO = MerchantAuthCopier.INSTANCE.copying(merchantAuth);
        if (merchantAuthDTO == null){
            return null;
        }
        merchantAuthDTO.setMerchant(this.merchantService.getMerchantByUserId(userId));
        return merchantAuthDTO;
    }

    @Override
    public void certificationAudit(Long id, String state,String message ) {
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = AssertUtils.available(this.getById(id), "商家认证不存在");
        merchantAuth.setReviewStatus(state);
        merchantAuth.setReviewTime(new Date());
        merchantAuth.setReviewer(userId.toString());
        merchantAuth.setMessage(message);
        this.updateById(merchantAuth);
        if (StringUtils.notEmpty(state) && "1".equals(state)){
            this.userService.updateUserRole(merchantAuth.getApplicant());
        }
    }
}
