package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderReturnApplyIdsDTO;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderReturnApplyMapper;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderReturnApply;
import com.chelvc.cloud.vehicle.server.service.IOmsOrderReturnApplyService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 订单退货申请Service业务层处理
 * 
 * @author liude
 * @date 2023-11-08
 */
@Service
public class OmsOrderReturnApplyServiceImpl extends ServiceImpl<OmsOrderReturnApplyMapper, OmsOrderReturnApply> implements
        IOmsOrderReturnApplyService, com.chelvc.cloud.vehicle.api.service.IOmsOrderReturnApplyService
{
    @Autowired
    private OmsOrderReturnApplyMapper omsOrderReturnApplyMapper;

    /**
     * 查询订单退货申请
     * 
     * @param id 订单退货申请主键
     * @return 订单退货申请
     */
    @Override
    public OmsOrderReturnApply selectOmsOrderReturnApplyById(Long id)
    {
        return omsOrderReturnApplyMapper.selectOmsOrderReturnApplyById(id);
    }

    /**
     * 查询订单退货申请列表
     * 
     * @param omsOrderReturnApply 订单退货申请
     * @return 订单退货申请
     */
    @Override
    public List<OmsOrderReturnApply> selectOmsOrderReturnApplyList(OmsOrderReturnApply omsOrderReturnApply)
    {
        return omsOrderReturnApplyMapper.selectOmsOrderReturnApplyList(omsOrderReturnApply);
    }

    /**
     * 新增订单退货申请
     * 
     * @param omsOrderReturnApply 订单退货申请
     * @return 结果
     */
    @Override
    public int insertOmsOrderReturnApply(OmsOrderReturnApply omsOrderReturnApply)
    {
        omsOrderReturnApply.setCreateTime(new Date());
        return omsOrderReturnApplyMapper.insertOmsOrderReturnApply(omsOrderReturnApply);
    }

    /**
     * 修改订单退货申请
     * 
     * @param omsOrderReturnApply 订单退货申请
     * @return 结果
     */
    @Override
    public int updateOmsOrderReturnApply(OmsOrderReturnApply omsOrderReturnApply)
    {
        return omsOrderReturnApplyMapper.updateOmsOrderReturnApply(omsOrderReturnApply);
    }

    /**
     * 批量删除订单退货申请
     * 
     * @param ids 需要删除的订单退货申请主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderReturnApplyByIds(Long[] ids)
    {
        return omsOrderReturnApplyMapper.deleteOmsOrderReturnApplyByIds(ids);
    }

    /**
     * 删除订单退货申请信息
     * 
     * @param id 订单退货申请主键
     * @return 结果
     */
    @Override
    public int deleteOmsOrderReturnApplyById(Long id)
    {
        return omsOrderReturnApplyMapper.deleteOmsOrderReturnApplyById(id);
    }

    @Override
    public Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(Long merchantId, Paging paging) {
        Page<OmsOrderReturnApply> page = this.lambdaQuery()
                .eq(OmsOrderReturnApply::getMerchantId, merchantId)
                .orderByAsc(OmsOrderReturnApply::getCreateTime).page(DatabaseContextHolder.page(paging));
        List<OmsOrderReturnApply> omsOrderReturnApplies = page.getRecords();
        List<OmsOrderReturnApplyIdsDTO> copy = ObjectUtils.copying(omsOrderReturnApplies, OmsOrderReturnApplyIdsDTO.class);
        return DatabaseContextHolder.pagination(page, copy);
    }

    @Override
    public Integer queryReturnOrderNum(Long merchantId) {
        LambdaQueryWrapper<OmsOrderReturnApply> lwq = Wrappers.lambdaQuery();
        lwq.eq(OmsOrderReturnApply::getMerchantId, merchantId);
        lwq.eq(OmsOrderReturnApply::getStatus, 0);
        return baseMapper.selectCount(lwq);
    }

    @Override
    public OmsOrderReturnApplyDTO getOrderReturnApplyInfo(Long id) {
        OmsOrderReturnApply omsOrderReturnApply = omsOrderReturnApplyMapper.selectOmsOrderReturnApplyById(id);
        if (omsOrderReturnApply == null){
            return null;
        }
        return ObjectUtils.copying(omsOrderReturnApply, OmsOrderReturnApplyDTO.class);
    }
}
