package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.dto.MerchantRankCountDTO;
import com.chelvc.cloud.vehicle.api.dto.RankAwardRecordDTO;
import com.chelvc.cloud.vehicle.api.param.HandRankAwardParam;
import com.chelvc.cloud.vehicle.api.param.QueryRankAwardRecordParam;
import com.chelvc.cloud.vehicle.api.service.RankAwardRecordService;
import com.chelvc.cloud.vehicle.api.util.DateTimeUtils;
import com.chelvc.cloud.vehicle.server.dao.RankAwardRecordMapper;
import com.chelvc.cloud.vehicle.server.dao.RegionMapper;
import com.chelvc.cloud.vehicle.server.entity.RankAwardConfig;
import com.chelvc.cloud.vehicle.server.entity.RankAwardRecord;
import com.chelvc.cloud.vehicle.server.entity.Region;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.RankAwardConfigService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.RankAwardRecordService.class)
public class RankAwardRecordServiceImpl extends ServiceImpl<RankAwardRecordMapper, RankAwardRecord> implements RankAwardRecordService {

    @DubboReference
    UserService userService;
    private final RankAwardConfigService rankAwardConfigService;
    private final RegionMapper regionMapper;
    private final MerchantService merchantService;
    private final CategoryService categoryService;

    @Override
    public Pagination<RankAwardRecordDTO> queryPageList(QueryRankAwardRecordParam param) {
        Paging paging = param.getPaging();
        Page<RankAwardRecordDTO> page = new Page<>(paging.getNumber(), paging.getSize());
        QueryWrapper<RankAwardConfig> wrapper = Wrappers.query();
        wrapper.eq(param.getType() != null, "r.type", param.getType());
        wrapper.eq(param.getPlatformType() != null, "r.platform_type", param.getPlatformType());
        wrapper.eq(param.getPeriod() != null, "r.period", param.getPeriod());
        wrapper.orderByDesc("r.create_time");
        IPage<RankAwardRecordDTO> result = baseMapper.queryPageList(page, wrapper);
        List<RankAwardRecordDTO> records = result.getRecords();
        if (!CollectionUtils.isEmpty(records) && param.getPlatformType() != null && param.getPlatformType() != 2) {
            List<Long> userIds =
                    records.stream().filter(e -> e.getPlatformType() != 2).map(RankAwardRecordDTO::getUserId).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(userIds)) {
                List<UserDTO> users = this.userService.listUsers(userIds);
                Map<Long, UserDTO> userMap = ObjectUtils.isEmpty(users) ? Collections.emptyMap() :
                        users.stream().collect(Collectors.toMap(UserDTO::getId, user -> user));
                for (RankAwardRecordDTO dto : records) {
                    Integer platformType = dto.getPlatformType();
                    if (platformType != 2) {
                        UserDTO userDTO = userMap.get(dto.getUserId());
                        dto.setName(userDTO == null ? "" : userDTO.getNickname());
                    }
                }
            }
        }
        return Pagination.<RankAwardRecordDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    @Override
    public void handOutPrize(HandRankAwardParam param) {
        Long awardId = param.getAwardId();
        RankAwardConfig config = rankAwardConfigService.queryById(awardId);
        if (config == null) {
            throw new ResourceUnavailableException("参数错误");
        }
        String period = param.getPeriod();
        Long userId = param.getUserId();
        LambdaQueryWrapper<RankAwardRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(RankAwardRecord::getUserId, userId);
        wrapper.eq(RankAwardRecord::getType, config.getType());
        wrapper.eq(RankAwardRecord::getPlatformType, config.getPlatformType());
        wrapper.eq(RankAwardRecord::getPeriod, period);
        List<RankAwardRecord> rankAwardRecords = baseMapper.selectList(wrapper);
        if (!CollectionUtils.isEmpty(rankAwardRecords)) {
            throw new ResourceUnavailableException("本次奖励已发放");
        }
        //TODO  发放奖励

        RankAwardRecord record = new RankAwardRecord();
        record.setUserId(userId);
        //TODO  比例转成具体金额
        record.setBonus(new BigDecimal(config.getRatio()));
        record.setRegionLevel(config.getRegionLevel());
        record.setPeriod(period);
        record.setType(config.getType());
        record.setPlatformType(config.getPlatformType());
        record.setAwardTime(LocalDateTime.now());
        record.setCreateTime(LocalDateTime.now());
        baseMapper.insert(record);
    }

    @Override
    public void cityAwardRank() {
        Integer level = 2;
        List<Integer> ids = regionMapper.queryIdsByLevel(level);
        List<RankAwardConfig> rankAwardConfigs = rankAwardConfigService.queryList(2, 2, level);
        if (CollectionUtils.isEmpty(rankAwardConfigs)) {
            return;
        }
        Map<Integer, RankAwardConfig> map =
                rankAwardConfigs.stream().collect(Collectors.toMap(RankAwardConfig::getRank, (p) -> p));
        int limit = rankAwardConfigs.size();
        //查询一级分类
        List<Long> categoryIds = categoryService.queryFirstLevelIds();
        if(categoryIds == null){
            return;
        }
        for (Integer id : ids) {
            for (Long categoryId : categoryIds) {
                List<RankAwardRecord> list = new ArrayList<>();
                List<MerchantRankCountDTO> result = merchantService.rankMerchants(id, limit, level, categoryId);
                BigDecimal totalCommissionAmount = merchantService.queryTotalCommissionAmountByLastMonth(id, level, categoryId);
                if (totalCommissionAmount == null || totalCommissionAmount.compareTo(BigDecimal.ZERO) == 0) {
                    break;
                }
                //查询区县已发出的奖励
                BigDecimal totalAmount = baseMapper.queryAmountByRegion(id, DateTimeUtils.getLastYearMonth());
                if (totalAmount != null && totalAmount.compareTo(BigDecimal.ZERO) > 0) {
                    totalCommissionAmount = totalCommissionAmount.subtract(totalAmount);
                }
                for (int i = 1; i <= result.size(); i++) {
                    MerchantRankCountDTO merchantRankCountDTO = result.get(i);
                    RankAwardConfig config = map.get(i);
                    BigDecimal bonus =
                            totalCommissionAmount.multiply(new BigDecimal(config.getCommissionRatio()).divide(new BigDecimal("100"), 2, RoundingMode.FLOOR))
                                    .multiply(new BigDecimal(config.getRatio()).divide(new BigDecimal("100"), 2,
                                            RoundingMode.FLOOR));
                    RankAwardRecord rankAwardRecord = new RankAwardRecord();
                    rankAwardRecord.setRank(i);
                    rankAwardRecord.setUserId(merchantRankCountDTO.getUserId());
                    rankAwardRecord.setBonus(bonus);
                    rankAwardRecord.setPeriod(DateTimeUtils.getLastYearMonth());
                    rankAwardRecord.setType(2);
                    rankAwardRecord.setPlatformType(2);
                    rankAwardRecord.setRegionLevel(level);
                    rankAwardRecord.setStatus(0);
                    rankAwardRecord.setAwardTime(LocalDateTime.now());
                    rankAwardRecord.setCreateTime(LocalDateTime.now());
                    list.add(rankAwardRecord);
                }
                baseMapper.batchInsert(list);
            }
        }
    }

    @Override
    public void districtAwardRank() {
        Integer level = 3;
        List<RankAwardConfig> rankAwardConfigs = rankAwardConfigService.queryList(2, 2, level);
        if (CollectionUtils.isEmpty(rankAwardConfigs)) {
            return;
        }
        //查询一级分类
        List<Long> categoryIds = categoryService.queryFirstLevelIds();
        if(categoryIds == null){
            return;
        }
        Map<Integer, RankAwardConfig> map =
                rankAwardConfigs.stream().collect(Collectors.toMap(RankAwardConfig::getRank, (p) -> p));
        LambdaQueryWrapper<Region> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Region::getLevel, level);
        Integer total = regionMapper.selectCount(wrapper);
        int pageNum = 1;
        int pageSize = 500;
        int pages = 1;
        if (total > pageSize) {
            pages = total % pageSize != 0 ? (total / pageSize) + 1 : total / pageSize;
        }
        while (pageNum <= pages) {
            List<RankAwardRecord> list = new ArrayList<>();
            List<Integer> ids = regionMapper.queryPageIdsByLevel(level, pageNum, pageSize);
            int limit = rankAwardConfigs.size();
            for (Integer id : ids) {
                for (Long categoryId : categoryIds) {
                    List<MerchantRankCountDTO> result = merchantService.rankMerchants(id, limit, level, categoryId);
                    BigDecimal totalCommissionAmount = merchantService.queryTotalCommissionAmountByLastMonth(id, level, categoryId);
                    if (totalCommissionAmount == null || totalCommissionAmount.compareTo(BigDecimal.ZERO) == 0) {
                        break;
                    }
                    for (int i = 1; i <= result.size(); i++) {
                        MerchantRankCountDTO merchantRankCountDTO = result.get(i);
                        RankAwardConfig config = map.get(i);
                        BigDecimal bonus =
                                totalCommissionAmount.multiply(new BigDecimal(config.getCommissionRatio()).divide(new BigDecimal("100"), 2, RoundingMode.FLOOR))
                                        .multiply(new BigDecimal(config.getRatio()).divide(new BigDecimal("100"), 2,
                                                RoundingMode.FLOOR));
                        RankAwardRecord rankAwardRecord = new RankAwardRecord();
                        rankAwardRecord.setRank(i);
                        rankAwardRecord.setUserId(merchantRankCountDTO.getUserId());
                        rankAwardRecord.setBonus(bonus);
                        rankAwardRecord.setPeriod(DateTimeUtils.getLastYearMonth());
                        rankAwardRecord.setType(2);
                        rankAwardRecord.setPlatformType(2);
                        rankAwardRecord.setRegionLevel(level);
                        rankAwardRecord.setStatus(0);
                        rankAwardRecord.setAwardTime(LocalDateTime.now());
                        rankAwardRecord.setCreateTime(LocalDateTime.now());
                        list.add(rankAwardRecord);
                    }
                }
            }
            baseMapper.batchInsert(list);
            pageNum += 1;
        }
    }
}
