package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.server.dao.ProfitRatioConfigMapper;
import com.chelvc.cloud.vehicle.server.dao.UserProfitRatioMapper;
import com.chelvc.cloud.vehicle.server.entity.ProfitRatioConfig;
import com.chelvc.cloud.vehicle.server.entity.UserProfitRatio;
import com.chelvc.cloud.vehicle.server.service.UserProfitRatioService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserProfitRatioServiceImpl extends ServiceImpl<UserProfitRatioMapper, UserProfitRatio> implements UserProfitRatioService {
    private final ProfitRatioConfigMapper profitRatioConfigMapper;

    @Override
    public int queryRatio(Long inviteUserId, Integer type) {
        LambdaQueryWrapper<UserProfitRatio> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(UserProfitRatio::getUserId, inviteUserId);
        wrapper.eq(UserProfitRatio::getType, type);
        UserProfitRatio userProfitRatio = baseMapper.selectOne(wrapper);
        if(userProfitRatio == null){
            UserProfitRatio init = init(inviteUserId, type);
            return init.getRatio();
        }
        return userProfitRatio.getRatio();
    }

    @Override
    public UserProfitRatio selectOne(Long userId, Integer type) {
        LambdaQueryWrapper<UserProfitRatio> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(UserProfitRatio::getUserId, userId);
        lambdaQuery.eq(UserProfitRatio::getType, type);
        UserProfitRatio userProfitRatio = baseMapper.selectOne(lambdaQuery);
        if (userProfitRatio == null) {
            return init(userId, type);
        }
        return userProfitRatio;
    }

    @Override
    public int update(UserProfitRatio userProfitRatio) {
        return baseMapper.update(userProfitRatio, null);
    }

    public UserProfitRatio init(Long userId, Integer type){
        LambdaQueryWrapper<ProfitRatioConfig> config = Wrappers.lambdaQuery();
        config.eq(ProfitRatioConfig::getType, type);
        config.eq(ProfitRatioConfig::getStatus, 0);
        config.orderByAsc(ProfitRatioConfig::getRatio);
        List<ProfitRatioConfig> list = profitRatioConfigMapper.selectList(config);
        int ratio = 0;
        if(CollectionUtils.isNotEmpty(list)){
            ratio = list.get(0).getRatio();
        }
        UserProfitRatio userProfitRatio = new UserProfitRatio();
        userProfitRatio.setRatio(ratio);
        userProfitRatio.setUserId(userId);
        userProfitRatio.setType(type);
        userProfitRatio.setCreateTime(LocalDateTime.now());
        userProfitRatio.setUpdateTime(LocalDateTime.now());
        userProfitRatio.setUpdater(PlatformConstant.PLATFORM_ID);
        baseMapper.insert(userProfitRatio);
        return userProfitRatio;
    }
}
